import fluid.command.ICommand;
import fluid.command.IAsynchronousCommand;
import mx.events.EventDispatcher;
import fluid.events.CommandEvent;
import fluid.logger.Logger;


/**
 * -----------------------------------------------------------------
 * Copyright (c) 2000-2006 Fluid, Inc. All Right Reserved.
 * This software is the proprietary information of Fluid, Inc.
 * Use is subject to strict licensing terms.
 * -----------------------------------------------------------------
 *
 * Provides support for implementing asynchronous commands. Enforces execution
 * of a single invocation at a time. Subclasses should not override 
 * <code>execute</code>, but rather implement <code>doExecute</code>.
 * <code>notifyComplete</code> is provided as a convenience for subclasses to
 * use.
 * 
 * @version $Revision: 1.4 $
 * @author $Author: ward $
 * 
 * @author Ward Ruth
 */
 
class fluid.command.AbstractAsynchronousCommand
		implements ICommand, IAsynchronousCommand
{

	private static var inited : Boolean = initializeEvents();
	
	private var _isExecuting : Boolean = false;
	
	/**	To be implemented by IConcreteCommands */
	public var doExecute : Function;
	
	/**	For EventDispatcher mix-in */
	private var dispatchEvent : Function;
	
	
	/**
	 *	Private constructor -- this is an "Abstract" class
	 */
	private function AbstractAsynchronousCommand()
	{
		Logger.logDebug( "AbstractAsynchronousCommand constructor", "command" );
	}
	
	public function toString() : String
	{
		return "[fluid.command.AbstractAsynchronousCommand]";	
	}
	
	/**
	 *	Subclasses of AbstractAsynchronousCommand should <strong>NOT</strong>
	 *	override this method. AbstractAsynchronousCommand will call the template
	 *	method <code>doExecute</code> in subclasses. Calling 
	 *	<code>execute</code> puts AbstractAsynchronousCommand in an executing 
	 *	state. Upon completion of the asynchronous operation, the command will 
	 *	broadcast a CommandEvent indicating the success or failure of the 
	 *	operation. Calling <code>execute</code> before the current operation has 
	 *	completed is considered an error, and will cause the command to 
	 *	immediately broadcast a failure CommandEvent.
	 */
	public function execute() : Void 
	{
		Logger.logDebug( this + ".execute(), isExecuting = " + _isExecuting, "command" );
		if( _isExecuting )
		{
			notifyComplete( false );
		}
		else
		{
			_isExecuting = true;
			doExecute();
		}
	}
	
	public function isExecuting() : Boolean
	{
		return _isExecuting;
	}
	
	//	Implementing IObservable
	
	public function addEventListener( eventType : String, handler : Object ) : Void 
	{
	}

	public function removeEventListener( eventType : String, handler : Object ) : Void 
	{
	}
	
	/**
	 *	Convenience method for subclasses. Will broadcast a CommandEvent with
	 *	a success accessor property matching the value passed in. Finishes
	 *	execution of the command.
	 *	
	 *	@param success	success or failure of the command (context specific)
	 */	
	private function notifyComplete( success : Boolean ) : Void
	{
		dispatchEvent( new CommandEvent( this, success ) );
		_isExecuting = false;
		Logger.logDebug( this + ".notifyComplete( " + success + " ), isExecuting = " + _isExecuting, "command" );
	}
	
	private static function initializeEvents() : Boolean
	{
		EventDispatcher.initialize( AbstractAsynchronousCommand.prototype );
		return true;	
	}

}