import fluid.command.AbstractAsynchronousCommand;
import fluid.command.IAsynchronousCommand;
import fluid.command.IConcreteCommand;
import fluid.command.ICommand;
import fluid.events.CommandEvent;


/**
 * -----------------------------------------------------------------
 * Copyright (c) 2000-2006 Fluid, Inc. All Right Reserved.
 * This software is the proprietary information of Fluid, Inc.
 * Use is subject to strict licensing terms.
 * -----------------------------------------------------------------
 *
 * Composite command implementation. Provides a simple way to execute a chain
 * of commands in sequence. CompositeCommand will behave synchronously if all
 * of it's child commands are synchronous, otherwise it will behave 
 * asynchronously.
 * 
 * @version $Revision: 1.3 $
 * @author $Author: aguldman $
 * 
 * @author Ward Ruth
 */
 
class fluid.command.CompositeCommand 
		extends AbstractAsynchronousCommand 
		implements IConcreteCommand 
{
	/**	the child commands */
	private var _commands : Array;
	
	/** pointer to the currently executing child command */
	private var _currentCommand : ICommand;
	
	/**	index of the curently executing child command in the _commands Array */
	private var _commandIndex : Number = -1;
	
	/**	CompositeCommand determines this based on it's children */
	private var _isExecutingAsynchronously : Boolean = false;
	
	
	function CompositeCommand() 
	{
		_commands = new Array();
	}

	/**
	 *	Determine if the composite is executing synchronously or asynchronously,
	 *	and begin executing child commands in sequence.
	 */
	private function doExecute() : Void 
	{
		_isExecutingAsynchronously = areAnyCommandsAsynchronous();
		_commandIndex = -1;
		executeNext();
	}
	
	/**
	 *	Add a child command to the composites command stack. Note that it is
	 *	legal to do this while the composite is executing.
	 *	
	 *	@param command	a child command
	 */
	public function add( command : ICommand ) : Void 
	{
		_commands.push( command );
	}

	/**
	 *	Remove a child command from the composites command stack. This is
	 *	evaluated on a pointer basis. <code>remove</code> will fail silently if
	 *	you attempt to remove the currently executing command.
	 *	
	 *	@param command	a child command
	 */
	public function remove( command : ICommand ) : Void 
	{
		//	Doesn't make sense to be able to remove the currently executing command
		if( command == _currentCommand )
		{
			return;
		}
		
		var ix : Number = _commands.length;
		
		while( ix-- )
		{
			if( ICommand( _commands[ ix ] ) == ICommand )
			{
				_commands.splice( ix, 1 );
			}	
		}
	}
	
	/**
	 *	Retrieves the child command at the specificed index.
	 *	
	 *	@param n	index of child to retrieve
	 *	@return		the child at the specified index
	 */
	public function getChildAt( n : Number ) : ICommand
	{
		return ICommand( _commands[ n ] );
	}
	
	/**
	 *	Number of child commands in the composite
	 *	
	 *	@return	the number of children
	 */
	public function getNumChildren() : Number
	{
		return _commands.length;
	}

	public function addEventListener(eventType : String, handler : Object) : Void 
	{
	}

	public function removeEventListener(eventType : String, handler : Object) : Void 
	{
	}
	
	/**
	 *	The composite is registered as a completion listener for the currently
	 *	executing child if it is an asynchronous child. If an asynchronous child
	 *	fails then execution of any remaining children is aborted and the 
	 *	composite broadcasts a failure CommandEvent. Otherwise
	 *	<code>executeNext</code> is called.
	 *	
	 *	@param evt	CommandEvent broadcast from the current child
	 */
	private function commandComplete( evt : CommandEvent ) : Void
	{
		var completedCommand : IAsynchronousCommand = IAsynchronousCommand( evt.target );
		completedCommand.removeEventListener( CommandEvent.TYPE, this );
		
		if( evt.success )
		{
			executeNext();
		}
		else
		{
			notifyComplete( false );
		}
	}
	
	/**
	 *	Executes next child command if there is one. Otherwise if executing
	 *	synchronously broadcasts a success CommandEvent.
	 */
	private function executeNext() : Void
	{
		if( ++_commandIndex >= _commands.length )
		{
			if( _isExecutingAsynchronously )
			{
				notifyComplete( true );
			}
		}
		else
		{
			_currentCommand = ICommand( _commands[ _commandIndex ] );
			
			if( _currentCommand instanceof IAsynchronousCommand )
			{
				IAsynchronousCommand( _currentCommand ).addEventListener( CommandEvent.TYPE, this );
				_currentCommand.execute();
			}
			else
			{
				_currentCommand.execute();
				executeNext();
			}
		}
	}
	
	/**
	 *	Internal utility method to determine if any children are asynchronous.
	 *	This is called once in response to <code>execute</code> being called
	 *	on the composite, and determines whether the composite broadcasts a
	 *	CommandEvent after executing all of it's children.
	 *	
	 *	@return	true if any children are asynchronous when composite is executed
	 */
	private function areAnyCommandsAsynchronous() : Boolean
	{
		var ix : Number = _commands.length;
		
		while( ix-- )
		{
			if( _commands[ ix ] instanceof IAsynchronousCommand )
			{
				return true;	
			}	
		}
		
		return false;	
	}
	
}
